﻿using UnityEngine;
using System.Collections;

public class PlayerScript : MonoBehaviour
{
    public GameObject deathPrefab;
    public static PlayerScript Self { get; private set; }

    public KeyCode rightMovement = KeyCode.RightArrow;
    public KeyCode leftMovement = KeyCode.LeftArrow;
    public KeyCode upMovement = KeyCode.UpArrow;

    public int lives = 3;
    public Texture heartTexture;

    public float minMobDistance = 6.2f;
    public float maxMobDistance = 12.0f;
    public float maxMobDistanceStep = 0.5f;

    public float acceleration = 0.025f;
    public float decceleration = 0.0025f;
    public float maxHorizontalSpeed = 0.16f;
    public float minHorizontalSpeed = 0.01f;
    public float jumpSpeed = 0.022f;
    public float gravity = 0.0036f;

    private int framesFromLastFloorTouch = 999;
    const int FRAMES_THAT_LEFT_YOU_ACT = 8;
    private static MobScript mob;

    public Vector2 currentSpeed;

    int score;
    const int SCORE_FRAME = 25;
    int frame = 0;

    public AnimationScript painAnimation;

    public Material runMaterial;
    public Material epicRunMaterial;
    public Material jumpMaterial;
    public Material fallMaterial;

    int frameLock = int.MaxValue;
    int invincibility = 0;


    void DoAnimation(Material animation, int framelock = int.MaxValue)
    {
        MeshRenderer mr = GetComponent<MeshRenderer>();
        if (!mr.material.name.StartsWith(animation.name))
        {
            frame = 0;
            frameLock = framelock;
            mr.material = animation;
        }
    }


    bool CanAct()
    {
        return lives > 0 && framesFromLastFloorTouch <= FRAMES_THAT_LEFT_YOU_ACT;
    }

    void Awake()
    {
        Self = this;
        mob = GameObject.Find("Mob").GetComponent<MobScript>();
    }
	
	// Update is called once per frame
	void Update () {
        if (lives <= 0)
        {
            Instantiate(deathPrefab, this.transform.position, this.transform.rotation);
            Destroy(this.gameObject);
        }
        frame++;
        if (frame % SCORE_FRAME == 0)
        {
            float delta = GetNormalizedDistanceToMob();
            score += (int)(delta * delta * 20.0f);
            score++;
        }

        if (currentSpeed.y > jumpSpeed && framesFromLastFloorTouch > 0)
        {
            DoAnimation(jumpMaterial, 20);
        }
		else if (currentSpeed.y < -gravity && framesFromLastFloorTouch > 0)
        {
            DoAnimation(fallMaterial);
        }
        else if (GetNormalizedDistanceToMob() > 0.95f)
        {
            DoAnimation(epicRunMaterial);
        }
        else
        {
            DoAnimation(runMaterial);
        }

		if (Mathf.Abs(currentSpeed.y) < jumpSpeed)
        	GetComponent<MeshRenderer>().material.SetFloat("_Frame", (float)Mathf.Min((int)((float)frame * 0.5f * Mathf.Pow((GetNormalizedDistanceToMob() - 0.1f), 0.25f)), frameLock));
		else
			GetComponent<MeshRenderer>().material.SetFloat("_Frame", (float)Mathf.Min((int)((float)frame), frameLock));

        if (lives > 0) {
            if (currentSpeed.x > maxHorizontalSpeed)
            {
                currentSpeed.x = maxHorizontalSpeed;
            }
            else if (currentSpeed.x < minHorizontalSpeed)
            {
                currentSpeed.x = minHorizontalSpeed;
            }

            if (Input.GetKeyUp(rightMovement))
            {
                currentSpeed.x += acceleration;
            }
            else if (Input.GetKey(leftMovement))
            {
                currentSpeed.x -= acceleration;
            }
            currentSpeed.x -= decceleration;
            if (CanAct())
            {
                if (Input.GetKey(upMovement))
                {
                    currentSpeed.y += jumpSpeed;
                    if (framesFromLastFloorTouch == 0)
                    {
                        currentSpeed.y += 10.0f * jumpSpeed;
                    }
                }
            }
        }
        else
        {
            currentSpeed.x = 0;
        }

        if (this.transform.position.x < mob.transform.position.x + minMobDistance * 0.5f)
        {
            currentSpeed.x += acceleration;
        }
        else if (this.transform.position.x > mob.transform.position.x + maxMobDistance)
        {
            float k = this.transform.position.x - (mob.transform.position.x + maxMobDistance);
            int k2 = (int)(k / maxMobDistanceStep) + 1;

			currentSpeed.x -= decceleration * k2 * 0.2f;
        }


        currentSpeed.y -= gravity;
        framesFromLastFloorTouch++;        

        this.transform.Translate(currentSpeed.x, currentSpeed.y, 0);

        if (invincibility > 0)
        {
            invincibility--;
        }
        if (invincibility < 90)
        {
            if (invincibility % 4 == 0)
            {
                GetComponent<MeshRenderer>().enabled = true;
            }
            else
            {
                GetComponent<MeshRenderer>().enabled = false;
            }
        }
	}

    void OnCollisionStay2D(Collision2D info)
    {
        if (info.gameObject.tag == "Terrain")
        {
            framesFromLastFloorTouch = 0;
            currentSpeed.y = 0;
        }
    }

    void OnCollisionEnter2D(Collision2D info)
    {
		if (info.collider.gameObject.name == "Portal")
			Application.LoadLevel ("Credits");

        ObstacleTriggerScript ots = info.gameObject.GetComponent<ObstacleTriggerScript>();
        if (ots != null)
        {
            ots.Trigger();
            Destroy(ots.gameObject);
        }
        DeathZone death = info.gameObject.GetComponentInChildren<DeathZone>();
        if (death != null)
        {
            if (death.animation == "Wall")
			{
				SoundPlayer.Self.Play (0);
                lives = 0;
            }
            else if (death.animation == "Dashable")
            {
                if (GetNormalizedDistanceToMob() < 0.95f)
                {
                    if (invincibility <= 0)
					{
						SoundPlayer.Self.Play (0);
                        lives--;
                        invincibility = 120;
                    }
                }
            }
            else
            {
                if (invincibility <= 0)
                {
					SoundPlayer.Self.Play (0);
                    lives--;
                    invincibility = 120;
                }
            }
            Destroy(info.collider);
            painAnimation.Execute(this.GetComponent<MeshRenderer>());
        }
		
		ColumnScript cs = info.gameObject.GetComponent<ColumnScript>();
        if (cs != null)
        {
            cs.Destroy();
        }
    }

    internal float GetNormalizedDistanceToMob()
    {
        float distance = this.transform.position.x - mob.transform.position.x;
        return distance / maxMobDistance;
    }

    void OnGUI()
    {
        GUI.Label(new Rect(0, 0, 200, 40), "Score: " + score);
        for (int i = 0; i < lives; i++)
        {
            GUI.Box(new Rect(Screen.width - 72 * i - 80, 5, 64, 64), heartTexture, GUIStyle.none);
        }
    }
}
